/*
	TUIO Max/MSP External - part of the reacTIVision project
	Copyright (c) 2005-2009 Martin Kaltenbrunner <mkalten@iua.upf.edu>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.
	 
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "TuioWrapper.h"
#include "ext.h"

std::list<TuioWrapper*> TuioClientList;

typedef struct _tuioclient
{
	t_object p_ob;								// object header
	void *message_outlet;
	void *refresh_outlet;						// outlet creation
	int port;									// the UDP port used by this instance
} t_tuioclient;

void *tuioclient_class;		// global pointer to the object class 

// these are prototypes for the methods that are defined below
void tuioclient_assist(t_tuioclient *x, void *b, long m, long a, char *s);
void *tuioclient_new(long n);
void tuioclient_free(t_tuioclient *x);

//--------------------------------------------------------------------------

int main(void)
{
	setup((t_messlist **)&tuioclient_class, (method)tuioclient_new, (method)tuioclient_free, (short)sizeof(t_tuioclient), 0L, A_DEFLONG, 0); 
	addmess((method)tuioclient_assist, "assist", A_CANT, 0);
	return 0;
}

//--------------------------------------------------------------------------

void tuioclient_assist(t_tuioclient *x, void *b, long m, long a, char *s) 
{
	if (m == ASSIST_OUTLET)
		switch (a) {	
		case 0:
			sprintf(s,"TUIO event output in list form");
			break;
		case 1:
			sprintf(s,"frame refresh bang");
			break;
		}
	else {
		sprintf(s,"input not used ...");
	}
}

//--------------------------------------------------------------------------

void *tuioclient_new(long n)		// n = port argument 
{
	t_tuioclient *x;
	x = (t_tuioclient *)newobject(tuioclient_class); // create a new instance 
	
	x->refresh_outlet = outlet_new(x,"bang");
	x->message_outlet = outlet_new(x,"list");

	x->port = (int)n;
	if(x->port<1024) x->port=3333;	

	TuioWrapper *wrapper = NULL;
	std::list<TuioWrapper*>::iterator iter;
	for (iter=TuioClientList.begin(); iter != TuioClientList.end(); iter++) {
		if ((*iter)->getPort()==x->port) {
			wrapper = *iter;
			break;
		}
	}
	
	if (wrapper==NULL) wrapper = new TuioWrapper(x->port);

	wrapper->addOutlet(x->message_outlet, x->refresh_outlet);
	
	if (!wrapper->isRunning()) {
		wrapper->start();
		if (wrapper->isRunning()) {
			TuioClientList.push_back(wrapper);
			post("TuioClient: started listening to UDP port %d",x->port);
		} else {
			delete wrapper;
			post("TuioClient: could not bind to UDP port %d",x->port);
		}
	}
	
	return(x); 
}

void tuioclient_free(t_tuioclient *x) {
	
	TuioWrapper *wrapper = NULL;
	std::list<TuioWrapper*>::iterator iter;
	for (iter=TuioClientList.begin(); iter != TuioClientList.end(); iter++) {
		if ((*iter)->getPort()==x->port) {
			wrapper = *iter;
			wrapper->removeOutlet(x->message_outlet, x->refresh_outlet);
			
			if (wrapper->outletCount()==0) {
				wrapper->stop();
				TuioClientList.erase(iter);
				delete wrapper;
				wrapper = NULL;
				post("TuioClient: stopped listening to UDP port %d",x->port);
			}
			break;
		}
	}	
}
